@@ -16,11 +16,12 @@ use libc::{syscall, SYS_ioctl};
 use nix::{
     errno::Errno,
     pty::Winsize,
+    sys::sendfile::sendfile64,
     unistd::{read, write},
 };
 
 use crate::{
-    compat::{fstatx, sendfile64, STATX_SIZE},
+    compat::{fstatx, STATX_SIZE},
     err2no,
     retry::retry_on_eintr,
 };
