@@ -122,7 +122,7 @@ fn syscall_link_handler(
         .as_ref()
         .map(|fd| fd.as_fd())
         .ok_or(Errno::EBADF)?;
-    assert!(old_path.base.is_empty()); // MUST_PATH!
+    assert!(old_path.base().is_empty()); // MUST_PATH!
 
     // SAFETY: Restrictions a la CONFIG_GRKERNSEC_LINK.
     if restrict_hardlinks {
@@ -131,13 +131,13 @@ fn syscall_link_handler(
 
     safe_linkat(
         PROC_FILE(),
-        &XPathBuf::from_self_fd(fd.as_raw_fd()),
+        &XPathBuf::from_self_fd(fd.as_raw_fd())?,
         new_path
             .dir
             .as_ref()
             .map(|fd| fd.as_fd())
             .ok_or(Errno::EBADF)?,
-        new_path.base,
+        new_path.base(),
         AtFlags::AT_SYMLINK_FOLLOW,
     )
     .map(|_| request.return_syscall(0))
@@ -179,6 +179,6 @@ fn safe_hardlink_source<Fd: AsFd>(fd: Fd, typ: FileType) -> Result<(), Errno> {
         &fd,
         XPath::empty(),
         AccessFlags::R_OK | AccessFlags::W_OK,
-        AtFlags::AT_EACCESS | AtFlags::AT_EMPTY_PATH,
+        crate::compat::AT_EACCESS | AtFlags::AT_EMPTY_PATH,
     )
 }
