@@ -12,7 +12,7 @@ use nix::{errno::Errno, sys::socket::SockaddrLike};
 
 use crate::{
     compat::{fstatx, STATX_INO},
-    config::MMAP_MIN_ADDR,
+    confine::is_valid_ptr,
     req::UNotifyEventRequest,
 };
 
@@ -45,7 +45,7 @@ pub(crate) fn handle_getsockname(
             // and return EINVAL as necessary.
             let len = i32::from_ne_bytes(buf);
             let len = libc::socklen_t::try_from(len).or(Err(Errno::EINVAL))?;
-            if args[1] < *MMAP_MIN_ADDR {
+            if !is_valid_ptr(args[1], request.scmpreq.data.arch) {
                 // address length is positive however address is NULL,
                 // return EFAULT.
                 return Err(Errno::EFAULT);
