@@ -12,14 +12,14 @@
 use libseccomp::ScmpNotifResp;
 use nix::errno::Errno;
 
-use crate::{config::MMAP_MIN_ADDR, req::UNotifyEventRequest, sysinfo::SysInfo};
+use crate::{confine::is_valid_ptr, req::UNotifyEventRequest, sysinfo::SysInfo};
 
 pub(crate) fn sys_sysinfo(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
         let req = request.scmpreq;
 
         // SAFETY: Check pointer against mmap_min_addr.
-        if req.data.args[0] < *MMAP_MIN_ADDR {
+        if !is_valid_ptr(req.data.args[0], req.data.arch) {
             return Err(Errno::EFAULT);
         }
 
