@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/log.rs: Simple logging on standard error using JSON lines
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon musl's src/time/__secs_to_tm.c which is:
 //   Copyright © 2005-2020 Rich Felker, et al.
 //   SPDX-License-Identifier: MIT
@@ -12,8 +12,8 @@
 use std::{
     fmt,
     io::Write,
-    os::fd::{BorrowedFd, RawFd},
-    sync::OnceLock,
+    os::fd::{AsFd, BorrowedFd, RawFd},
+    sync::{atomic::Ordering, OnceLock},
     thread::ThreadId,
     time::{SystemTime, UNIX_EPOCH},
 };
@@ -30,7 +30,7 @@ use serde_json::{Map, Value};
 use crate::{
     config::*,
     err::SydResult,
-    fs::is_active_fd,
+    fd::{is_active_fd, is_writable_fd},
     ioctl::{Ioctl, IoctlMap, IoctlName},
     ofd::{lock_fd, unlock_fd},
     proc::{proc_cmdline, proc_comm, proc_cwd, proc_tty},
@@ -77,6 +77,17 @@ pub(crate) static LOG_FD: std::sync::atomic::AtomicI32 = std::sync::atomic::Atom
 // Main thread identifier, used to decide on panic hook.
 pub(crate) static LOG_MAIN_TID: OnceLock<ThreadId> = OnceLock::new();
 
+// Validate log file descriptor.
+fn validate_log_fd<Fd: AsFd>(fd: Fd) -> Result<(), Errno> {
+    if !is_active_fd(&fd) {
+        return Err(Errno::EBADF);
+    }
+    if !is_writable_fd(fd).unwrap_or(false) {
+        return Err(Errno::EBADFD);
+    }
+    Ok(())
+}
+
 // Set this in main thread after namespace forks and other dances.
 pub(crate) fn log_init_main() -> SydResult<()> {
     LOG_MAIN_TID
@@ -133,6 +144,12 @@ macro_rules! emerg {
         if $crate::log_enabled!($crate::syslog::LogLevel::Emergent) {
             let timestamp = $crate::log::now();
             let mut map = serde_json::Map::new();
+            #[expect(clippy::disallowed_methods)]
+            if let Ok(id) = std::env::var($crate::config::ENV_ID) {
+                map.insert("id".to_string(), id.into());
+            }
+            let syd = nix::unistd::gettid().as_raw().into();
+            map.insert("syd".to_string(), serde_json::Value::Number(syd));
             $(
                 if let Ok(value) = serde_json::to_value($value) {
                     map.insert($key.to_string(), value);
@@ -154,6 +171,12 @@ macro_rules! alert {
         if $crate::log_enabled!($crate::syslog::LogLevel::Alert) {
             let timestamp = $crate::log::now();
             let mut map = serde_json::Map::new();
+            #[expect(clippy::disallowed_methods)]
+            if let Ok(id) = std::env::var($crate::config::ENV_ID) {
+                map.insert("id".to_string(), id.into());
+            }
+            let syd = nix::unistd::gettid().as_raw().into();
+            map.insert("syd".to_string(), serde_json::Value::Number(syd));
             $(
                 if let Ok(value) = serde_json::to_value($value) {
                     map.insert($key.to_string(), value);
@@ -175,6 +198,12 @@ macro_rules! crit {
         if $crate::log_enabled!($crate::syslog::LogLevel::Crit) {
             let timestamp = $crate::log::now();
             let mut map = serde_json::Map::new();
+            #[expect(clippy::disallowed_methods)]
+            if let Ok(id) = std::env::var($crate::config::ENV_ID) {
+                map.insert("id".to_string(), id.into());
+            }
+            let syd = nix::unistd::gettid().as_raw().into();
+            map.insert("syd".to_string(), serde_json::Value::Number(syd));
             $(
                 if let Ok(value) = serde_json::to_value($value) {
                     map.insert($key.to_string(), value);
@@ -196,6 +225,12 @@ macro_rules! error {
         if $crate::log_enabled!($crate::syslog::LogLevel::Err) {
             let timestamp = $crate::log::now();
             let mut map = serde_json::Map::new();
+            #[expect(clippy::disallowed_methods)]
+            if let Ok(id) = std::env::var($crate::config::ENV_ID) {
+                map.insert("id".to_string(), id.into());
+            }
+            let syd = nix::unistd::gettid().as_raw().into();
+            map.insert("syd".to_string(), serde_json::Value::Number(syd));
             $(
                 if let Ok(value) = serde_json::to_value($value) {
                     map.insert($key.to_string(), value);
@@ -217,6 +252,12 @@ macro_rules! warn {
         if $crate::log_enabled!($crate::syslog::LogLevel::Warn) {
             let timestamp = $crate::log::now();
             let mut map = serde_json::Map::new();
+            #[expect(clippy::disallowed_methods)]
+            if let Ok(id) = std::env::var($crate::config::ENV_ID) {
+                map.insert("id".to_string(), id.into());
+            }
+            let syd = nix::unistd::gettid().as_raw().into();
+            map.insert("syd".to_string(), serde_json::Value::Number(syd));
             $(
                 if let Ok(value) = serde_json::to_value($value) {
                     map.insert($key.to_string(), value);
@@ -238,6 +279,12 @@ macro_rules! notice {
         if $crate::log_enabled!($crate::syslog::LogLevel::Notice) {
             let timestamp = $crate::log::now();
             let mut map = serde_json::Map::new();
+            #[expect(clippy::disallowed_methods)]
+            if let Ok(id) = std::env::var($crate::config::ENV_ID) {
+                map.insert("id".to_string(), id.into());
+            }
+            let syd = nix::unistd::gettid().as_raw().into();
+            map.insert("syd".to_string(), serde_json::Value::Number(syd));
             $(
                 if let Ok(value) = serde_json::to_value($value) {
                     map.insert($key.to_string(), value);
@@ -259,6 +306,12 @@ macro_rules! info {
         if $crate::log_enabled!($crate::syslog::LogLevel::Info) {
             let timestamp = $crate::log::now();
             let mut map = serde_json::Map::new();
+            #[expect(clippy::disallowed_methods)]
+            if let Ok(id) = std::env::var($crate::config::ENV_ID) {
+                map.insert("id".to_string(), id.into());
+            }
+            let syd = nix::unistd::gettid().as_raw().into();
+            map.insert("syd".to_string(), serde_json::Value::Number(syd));
             $(
                 if let Ok(value) = serde_json::to_value($value) {
                     map.insert($key.to_string(), value);
@@ -280,6 +333,12 @@ macro_rules! debug {
         if $crate::log_enabled!($crate::syslog::LogLevel::Debug) {
             let timestamp = $crate::log::now();
             let mut map = serde_json::Map::new();
+            #[expect(clippy::disallowed_methods)]
+            if let Ok(id) = std::env::var($crate::config::ENV_ID) {
+                map.insert("id".to_string(), id.into());
+            }
+            let syd = nix::unistd::gettid().as_raw().into();
+            map.insert("syd".to_string(), serde_json::Value::Number(syd));
             $(
                 if let Ok(value) = serde_json::to_value($value) {
                     map.insert($key.to_string(), value);
@@ -326,7 +385,6 @@ impl Write for LockedWriter<'_> {
 
 /// Initializes Syslog global object, reading environment variables.
 #[cfg(feature = "log")]
-#[expect(clippy::cognitive_complexity)]
 pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Result<(), Errno> {
     use std::os::unix::ffi::OsStrExt;
 
@@ -345,6 +403,7 @@ pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resul
     // If ENV_LOG_FD is set, we parse it:
     // 1. negative fd is ok as a shorthand to disable logging.
     // 2. positive fd must be a valid fd or we bail with EBADF.
+    // 3. positive fd must be a writable fd or we bail with EBADFD.
     let fd = match std::env::var_os(ENV_LOG_FD) {
         None => default_log_fd,
         Some(val) => {
@@ -352,15 +411,13 @@ pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resul
             if fd >= 0 {
                 // SAFETY: fd only used after validation.
                 let fd = unsafe { BorrowedFd::borrow_raw(fd) };
-                if !is_active_fd(fd) {
-                    return Err(Errno::EBADF);
-                }
+                validate_log_fd(fd)?;
             }
             Some(fd)
         }
     };
     if let Some(fd) = fd {
-        LOG_FD.store(fd, std::sync::atomic::Ordering::Relaxed);
+        LOG_FD.store(fd, Ordering::Relaxed);
     }
 
     // Decide if we want a TTY-like console.
@@ -377,7 +434,7 @@ pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resul
     }
 
     // Record TTY information to an atomic for easy access.
-    LOG_TTY.store(tty, std::sync::atomic::Ordering::Relaxed);
+    LOG_TTY.store(tty, Ordering::Relaxed);
 
     // Allocate the syslog(2) ring buffer on the stack by default.
     // The size is architecture-dependent, see config.rs.
@@ -395,7 +452,7 @@ pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resul
             .or(Err(Errno::EINVAL))?;
         use_stack = false;
     }
-    init_global_syslog(logbuflen, fd, level, use_stack)?;
+    init_global_syslog(logbuflen, level, use_stack)?;
 
     // Finally let's make some noise!
     info!("ctx": "init", "op": "sing", "chapter": 24,
@@ -426,6 +483,7 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
     // If ENV_LOG_FD is set, we parse it:
     // 1. negative fd is ok as a shorthand to disable logging.
     // 2. positive fd must be a valid fd or we bail with EBADF.
+    // 3. positive fd must be a writable fd or we bail with EBADFD.
     let fd = match std::env::var_os(ENV_LOG_FD) {
         None => libc::STDERR_FILENO,
         Some(val) => {
@@ -433,14 +491,12 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
             if fd >= 0 {
                 // SAFETY: fd only used after validation.
                 let fd = unsafe { BorrowedFd::borrow_raw(fd) };
-                if !is_active_fd(fd) {
-                    return Err(Errno::EBADF);
-                }
+                validate_log_fd(fd)?;
             }
             fd
         }
     };
-    LOG_FD.store(fd, std::sync::atomic::Ordering::Relaxed);
+    LOG_FD.store(fd, Ordering::Relaxed);
 
     // Decide if we want a TTY-like console.
     let mut tty = std::env::var_os(ENV_FORCE_TTY).is_some();
@@ -455,14 +511,13 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
     }
 
     // Record TTY information to an atomic for easy access.
-    LOG_TTY.store(tty, std::sync::atomic::Ordering::Relaxed);
+    LOG_TTY.store(tty, Ordering::Relaxed);
 
     // Create a global syslog with ring.
-    init_global_syslog(0, Some(fd), level, true)?;
+    init_global_syslog(0, level, true)?;
 
     if let Some(sys) = global_syslog() {
-        // Now lock it immediately,
-        // so ring is unavailable.
+        // Lock it immediately, so ring is unavailable.
         sys.lock();
     }
 
@@ -481,7 +536,7 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
 
     // For "higher" severities, we add more contextual fields.
     let add_context = level.as_u8() <= crate::syslog::LogLevel::Notice.as_u8();
-    let tty = LOG_TTY.load(std::sync::atomic::Ordering::Relaxed);
+    let tty = LOG_TTY.load(Ordering::Relaxed);
 
     // If there's a "pid", we might add cmd/cwd, etc.
     // We remove and re-add to reorder for better visibility.
@@ -513,16 +568,6 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
         msg.insert("uid".to_string(), Uid::current().as_raw().into());
     }
 
-    // Add "id" = sandbox ID.
-    #[expect(clippy::disallowed_methods)]
-    if let Ok(id) = std::env::var(ENV_ID) {
-        msg.insert("id".to_string(), id.into());
-    }
-
-    // Add "syd" = current thread ID.
-    let syd = nix::unistd::gettid().as_raw().into();
-    msg.insert("syd".to_string(), Value::Number(syd));
-
     // Add ISO8601 date, fallback to timestamp.
     if let Ok(date) = format_iso8601(timestamp) {
         msg.insert("time".to_string(), date.into());
@@ -574,6 +619,7 @@ pub fn log_init(_default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resu
     // If ENV_LOG_FD is set, we parse it:
     // 1. negative fd is ok as a shorthand to disable logging.
     // 2. positive fd must be a valid fd or we bail with EBADF.
+    // 3. positive fd must be a writable fd or we bail with EBADFD.
     let fd = match std::env::var_os(ENV_LOG_FD) {
         None => default_log_fd,
         Some(val) => {
@@ -581,15 +627,13 @@ pub fn log_init(_default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resu
             if fd >= 0 {
                 // SAFETY: fd only used after validation.
                 let fd = unsafe { BorrowedFd::borrow_raw(fd) };
-                if !is_active_fd(fd) {
-                    return Err(Errno::EBADF);
-                }
+                validate_log_fd(fd)?;
             }
             Some(fd)
         }
     };
     if let Some(fd) = fd {
-        LOG_FD.store(fd, std::sync::atomic::Ordering::Relaxed);
+        LOG_FD.store(fd, Ordering::Relaxed);
     }
 
     // Decide if we want a TTY-like console.
@@ -606,13 +650,14 @@ pub fn log_init(_default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resu
     }
 
     // Record TTY information to an atomic for easy access.
-    LOG_TTY.store(tty, std::sync::atomic::Ordering::Relaxed);
+    LOG_TTY.store(tty, Ordering::Relaxed);
 
     // Determine the main FD for logging:
     // By default, we use stderr.
     // If ENV_LOG_FD is set, we parse it:
     // 1. negative fd is ok as a shorthand to disable logging.
     // 2. positive fd must be a valid fd or we bail with EBADF.
+    // 3. positive fd must be a writable fd or we bail with EBADFD.
     let fd = match std::env::var_os(ENV_LOG_FD) {
         None => default_log_fd,
         Some(val) => {
@@ -620,15 +665,13 @@ pub fn log_init(_default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resu
             if fd >= 0 {
                 // SAFETY: fd only used after validation.
                 let fd = unsafe { BorrowedFd::borrow_raw(fd) };
-                if !is_active_fd(fd) {
-                    return Err(Errno::EBADF);
-                }
+                validate_log_fd(fd)?;
             }
             Some(fd)
         }
     };
     if let Some(fd) = fd {
-        LOG_FD.store(fd, std::sync::atomic::Ordering::Relaxed);
+        LOG_FD.store(fd, Ordering::Relaxed);
     }
 
     // Finally let's make some noise!
@@ -652,17 +695,17 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
 #[expect(clippy::cognitive_complexity)]
 pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String, Value>) {
     // Setting LOG-FD to negative is OK to disable logging.
-    let fd = LOG_FD.load(std::sync::atomic::Ordering::Relaxed);
+    let fd = LOG_FD.load(Ordering::Relaxed);
     let fd = if fd < 0 {
         return; // Logging is disabled.
     } else {
-        // SAFETY: we trust log fd is valid.
+        // SAFETY: `LOG_FD` is valid for process lifetime.
         unsafe { BorrowedFd::borrow_raw(fd) }
     };
 
     // For "higher" severities, we add more contextual fields.
     let add_context = level.as_u8() <= crate::syslog::LogLevel::Notice.as_u8();
-    let tty = LOG_TTY.load(std::sync::atomic::Ordering::Relaxed);
+    let tty = LOG_TTY.load(Ordering::Relaxed);
 
     // If there's a "pid", we might add cmd/cwd, etc.
     // We remove and re-add to reorder for better visibility.
@@ -694,16 +737,6 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
         msg.insert("uid".to_string(), Uid::current().as_raw().into());
     }
 
-    // Add "id" = sandbox ID.
-    #[expect(clippy::disallowed_methods)]
-    if let Ok(id) = std::env::var(ENV_ID) {
-        msg.insert("id".to_string(), id.into());
-    }
-
-    // Add "syd" = current thread ID.
-    let syd = nix::unistd::gettid().as_raw().into();
-    msg.insert("syd".to_string(), Value::Number(syd));
-
     // Add ISO8601 date, fallback to timestamp.
     if let Ok(date) = format_iso8601(timestamp) {
         msg.insert("time".to_string(), date.into());
