@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/test/util.rs: Utilities for integration tests
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -28,12 +28,12 @@ use memchr::memmem;
 use nix::{
     errno::Errno,
     sys::{
-        resource::{setrlimit, Resource, RLIM_INFINITY},
+        resource::{Resource, RLIM_INFINITY},
         utsname::uname,
     },
     unistd::isatty,
 };
-use syd::config::*;
+use syd::{config::*, confine::confine_rlimit};
 
 #[derive(Debug)]
 pub struct TestError(pub String);
@@ -168,6 +168,13 @@ macro_rules! assert_status_panicked {
     }};
 }
 
+#[macro_export]
+macro_rules! assert_status_timeout_exceeded {
+    ($status:expr) => {{
+        assert_status_code!($status, 124);
+    }};
+}
+
 #[macro_export]
 macro_rules! assert_status_code_matches {
     ($status:expr, $($pattern:tt)+) => {{
@@ -232,6 +239,13 @@ macro_rules! assert_status_permission_denied {
     };
 }
 
+#[macro_export]
+macro_rules! assert_status_no_such_process {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::ESRCH);
+    };
+}
+
 #[macro_export]
 macro_rules! assert_status_broken_pipe {
     ($status:expr) => {
@@ -246,6 +260,20 @@ macro_rules! assert_status_busy {
     };
 }
 
+#[macro_export]
+macro_rules! assert_status_connection_refused {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::ECONNREFUSED);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_faulted {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::EFAULT);
+    };
+}
+
 #[macro_export]
 macro_rules! assert_status_hidden {
     ($status:expr) => {
@@ -260,6 +288,20 @@ macro_rules! assert_status_notdir {
     };
 }
 
+#[macro_export]
+macro_rules! assert_status_notty {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::ENOTTY);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_illegal_sequence {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::EILSEQ);
+    };
+}
+
 #[macro_export]
 macro_rules! assert_status_invalid {
     ($status:expr) => {
@@ -301,6 +343,19 @@ macro_rules! assert_status_killed {
     };
 }
 
+#[macro_export]
+macro_rules! assert_status_not_killed {
+    ($status:expr) => {
+        let code = $status.code().unwrap_or(127);
+        $crate::assert_ne!(
+            code,
+            128 + libc::SIGKILL,
+            "code:{code} status:{:?}",
+            $status
+        );
+    };
+}
+
 #[macro_export]
 macro_rules! assert_status_sigsys {
     ($status:expr) => {
@@ -428,6 +483,20 @@ macro_rules! skip_if_root {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_if_zfs {
+    () => {{
+        let cwd = open(".", OFlag::O_PATH | OFlag::O_DIRECTORY, Mode::empty()).unwrap();
+        if syd::compat::FsType::get(cwd).unwrap().is_zfs() {
+            eprintln!("Test does not run correctly on ZFS, skipping!");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        } else {
+            eprintln!("Test does not run on ZFS, continuing...");
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_iproute2 {
     () => {{
@@ -727,6 +796,40 @@ macro_rules! skip_unless_cap {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_if_landlock_abi_supported {
+    ($abi:expr) => {{
+        use std::process::Command;
+
+        // Run `syd-lock -A` to get the ABI version as exit code.
+        let status = Command::new(&*SYD_LOCK)
+            .arg("-A")
+            .stdin(Stdio::null())
+            .stdout(Stdio::null())
+            .status()
+            .expect("Failed to run syd-lock -A");
+
+        // SAFETY: We expect `syd-lock -A` to exit with a code.
+        let exit_code = status
+            .code()
+            .expect("Failed to get exit code from syd-lock -A");
+
+        if exit_code >= $abi {
+            eprintln!(
+                "Test is for Landlock ABI versions older than {}, skipping!",
+                $abi
+            );
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        } else {
+            eprintln!(
+                "Landlock ABI {} is NOT supported, proceeding with test...",
+                $abi
+            );
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_landlock_abi_supported {
     ($abi:expr) => {{
@@ -735,6 +838,8 @@ macro_rules! skip_unless_landlock_abi_supported {
         // Run `syd-lock -A` to get the ABI version as exit code.
         let status = Command::new(&*SYD_LOCK)
             .arg("-A")
+            .stdin(Stdio::null())
+            .stdout(Stdio::null())
             .status()
             .expect("Failed to run syd-lock -A");
 
@@ -853,6 +958,7 @@ pub static SYD_DNS: LazyLock<String> =
 
 /// Holds a `String` to run `syd-env`.
 /// Honours CARGO_BIN_EXE_syd-env environment variable.
+#[allow(dead_code)]
 pub static SYD_ENV: LazyLock<String> =
     LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-env").unwrap_or("syd-env".to_string()));
 
@@ -967,6 +1073,7 @@ pub static CI_BUILD: LazyLock<bool> = LazyLock::new(|| env::var("JOB_ID").ok().i
 pub static GL_BUILD: LazyLock<bool> = LazyLock::new(|| env::var("CI_PROJECT_ID").ok().is_some());
 
 /// A boolean which specifies whether standard error is a TTY.
+#[allow(dead_code)]
 pub static SYD_TEST_TTY: LazyLock<bool> =
     LazyLock::new(|| isatty(std::io::stderr()).unwrap_or(false));
 
@@ -1082,6 +1189,7 @@ impl Syd {
         self
     }
 
+    #[allow(dead_code)]
     pub fn quiet(&mut self) -> &mut Self {
         self.stderr(Stdio::null());
         self.stdout(Stdio::null());
@@ -1144,9 +1252,11 @@ pub fn syd() -> Syd {
         LazyLock::new(|| env::var_os("SYD_TEST_HELGRIND").is_some());
     let mut cmd = Syd::new("timeout");
     if check_timeout_foreground() {
+        // Use -v, not --verbose.
+        // Latter is not supported by Android.
+        cmd.arg("-v");
         cmd.arg("--foreground");
         cmd.arg("--preserve-status");
-        cmd.arg("--verbose");
     }
     cmd.arg("-sKILL");
     cmd.arg(env::var("SYD_TEST_TIMEOUT").unwrap_or("15m".to_string()));
@@ -1199,8 +1309,6 @@ pub fn syd() -> Syd {
     // Set logging level to warning.
     // Allow user to override.
     cmd.env(ENV_LOG, env::var(ENV_LOG).unwrap_or("warn".to_string()));
-    // Set log/verbose to false to avoid noisy test logs.
-    cmd.m("log/verbose:0");
     // Quiet TTY output to avoid noisy test logs.
     // Unless otherwise specified.
     if env::var_os(ENV_FORCE_TTY).is_none() {
@@ -1568,7 +1676,7 @@ pub fn check_strace_inject() -> bool {
 /// Enable coredumps.
 pub fn enable_coredumps() -> Result<(), Errno> {
     // Set both the soft and hard limits
-    setrlimit(Resource::RLIMIT_CORE, RLIM_INFINITY, RLIM_INFINITY)
+    confine_rlimit(Resource::RLIMIT_CORE, Some(RLIM_INFINITY))
 }
 
 /// Check if timeout --foreground is supported.
@@ -1616,14 +1724,20 @@ pub fn current_dir(base: bool) -> std::io::Result<PathBuf> {
 
 /// Shuffles a vector using the Fisher-Yates algorithm,
 /// utilizing random numbers obtained via `libc::rand`.
+/// On Android, falls back to `syd::rng::randint`.
 ///
 /// # Arguments
 /// * `vec` - A mutable reference to the vector to shuffle.
 pub fn shuffle_vec<T>(vec: &mut [T]) {
     let len = vec.len();
     for i in 0..len {
+        // Android does not define libc::rand().
+        #[cfg(not(target_os = "android"))]
         let r = unsafe { libc::rand() } as usize;
+        #[cfg(target_os = "android")]
+        let r = syd::rng::randint(usize::MIN..=usize::MAX).unwrap();
+
         let j = r % (len - i) + i;
-        vec.swap(i, j); // perform the swap
+        vec.swap(i, j); // Perform the swap.
     }
 }
