@@ -15,16 +15,26 @@ use memchr::arch::all::is_equal;
 use nix::errno::Errno;
 use syd::{
     confine::{lock_enabled, run_cmd},
-    landlock::{CompatLevel, Errata, RestrictSelfFlags, RulesetStatus, ABI},
+    err::err2no,
+    landlock::{
+        AddRuleError, AddRulesError, CompatLevel, CreateRulesetError, Errata, RestrictSelfError,
+        RestrictSelfFlags, RulesetError, RulesetStatus, ABI,
+    },
     landlock_policy::{LandlockPolicy, LANDLOCK_ACCESS_FS, LANDLOCK_ACCESS_NET},
     parsers::sandbox::parse_landlock_cmd,
     path::XPathBuf,
 };
 
-// Set global allocator to mimalloc.
-#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    not(target_os = "android"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
 #[global_allocator]
-static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
 
 // Set global allocator to tcmalloc if profiling is enabled.
 #[cfg(feature = "prof")]
@@ -250,10 +260,48 @@ syd::main! {
                 return Ok(ExitCode::FAILURE);
             }
         },
-        Err(error) => {
-            eprintln!("syd-lock: Landlock ABI {} unsupported: {error}", abi as i32);
-            return Ok(ExitCode::FAILURE);
+        Err(error) if policy.compat_level == Some(CompatLevel::BestEffort) => {
+            eprintln!("syd-lock: Landlock ABI {} is unsupported: {error}!", abi as i32);
+            eprintln!("syd-lock: Compatibility level is best effort, resuming...");
+            // fall-through.
         }
+        Err(
+            RulesetError::AddRules(AddRulesError::Fs(AddRuleError::AddRuleCall { source, .. }))
+            | RulesetError::AddRules(AddRulesError::Net(AddRuleError::AddRuleCall {
+                source, ..
+            }))
+            | RulesetError::AddRules(AddRulesError::Scope(AddRuleError::AddRuleCall {
+                source, ..
+            })),
+        ) => {
+            let errno = err2no(&source);
+            eprintln!("syd-lock: Landlock add rules error: {source}!");
+            return Err(errno.into());
+        }
+        Err(RulesetError::CreateRuleset(CreateRulesetError::CreateRulesetCall {
+            source, ..
+        })) => {
+            let errno = err2no(&source);
+            eprintln!("syd-lock: Landlock create ruleset error: {source}!");
+            return Err(errno.into());
+        }
+        Err(RulesetError::RestrictSelf(RestrictSelfError::SetNoNewPrivsCall {
+            source, ..
+        })) => {
+            let errno = err2no(&source);
+            eprintln!("syd-lock: Set no new privs error: {source}!");
+            return Err(errno.into());
+        }
+        Err(RulesetError::RestrictSelf(RestrictSelfError::RestrictSelfCall { source, .. })) => {
+            let errno = err2no(&source);
+            eprintln!("syd-lock: Landlock restrict self error: {source}!");
+            return Err(errno.into());
+        }
+        Err(source) => {
+            eprintln!("syd-lock: Landlock handle accesses error: {source}!");
+            return Err(Errno::ENOTSUP.into());
+        }
+
     };
 
     // Execute command, /bin/sh by default.
