@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/utils/syd-ofd.rs: Take a lock on a file, then execute into another program
 //
-// Copyright (c) 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon s6-setlock utility of skarnet s6 suite which is:
 //   Copyright (c) 2011-2025 Laurent Bercot <ska-skaware@skarnet.org>
 //   SPDX-License-Identifier: ISC
@@ -45,6 +45,22 @@ use syd::{
     timer::AlarmTimer,
 };
 
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    not(target_os = "android"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
+#[global_allocator]
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 // Signal handler function for SIGALRM.
 extern "C" fn handle_sigalrm(_: libc::c_int) {}
 
