@@ -35,20 +35,9 @@ use syd::{
     path::XPathBuf,
 };
 
-// Set global allocator to GrapheneOS allocator.
-#[cfg(all(
-    not(coverage),
-    not(feature = "prof"),
-    target_page_size_4k,
-    target_pointer_width = "64"
-))]
-#[global_allocator]
-static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
-
-// Set global allocator to tcmalloc if profiling is enabled.
-#[cfg(feature = "prof")]
-#[global_allocator]
-static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+// We intentionally do not use GrapheneOS allocator
+// in this binary to let it freely corrupt memory,
+// which is required for some of the POCs.
 
 syd::main! {
     use lexopt::prelude::*;
