@@ -18,44 +18,51 @@ use std::{
         fd::{AsFd, AsRawFd, BorrowedFd, FromRawFd},
         unix::ffi::OsStrExt,
     },
-    time::{Duration, Instant},
+    time::Instant,
 };
 
 use bitflags::bitflags;
 use data_encoding::HEXLOWER;
+use dur::Duration;
 use libseccomp::{ScmpAction, ScmpFilterContext, ScmpSyscall};
 use memchr::memchr3;
 use nix::{
     errno::Errno,
-    fcntl::OFlag,
+    fcntl::{open, OFlag},
     libc::{_exit, c_char, size_t, CLONE_FILES, ENOSYS, SIGCHLD, SIGKILL, SIGSYS},
-    mount::{mount, MsFlags},
+    mount::MsFlags,
     sched::{unshare, CloneFlags},
     sys::{
-        prctl::set_pdeathsig,
+        resource::Resource,
         signal::{sigprocmask, SigSet, SigmaskHow, Signal},
+        stat::Mode,
         wait::{Id, WaitPidFlag},
     },
-    unistd::{chdir, close, Gid, Pid, Uid},
+    unistd::{chdir, Gid, Pid, Uid},
 };
 
 use crate::{
-    compat::{pipe2_raw, waitid, MFdFlags, WaitStatus},
+    compat::{pipe2_raw, set_pdeathsig, waitid, MFdFlags, WaitStatus},
     config::{MINI_STACK_SIZE, *},
-    confine::{confine_mdwe, CLONE_NEWTIME, MS_NOSYMFOLLOW},
+    confine::{confine_mdwe, confine_rlimit, CLONE_NEWTIME},
     cookie::safe_memfd_create,
     debug,
     err::err2no,
-    fs::{pidfd_send_signal, safe_clone, seal_memfd_all, set_cloexec, set_nonblock},
+    fd::{close, pidfd_send_signal, seal_memfd_all, set_cloexec, set_nonblock},
+    fs::safe_clone,
     get_user_home, get_user_name,
     hash::SydHashSet,
     landlock::RulesetStatus,
     landlock_policy::LandlockPolicy,
     log::contains_ascii_unprintable,
     lookup::safe_copy_if_exists,
+    mount::{
+        api::MountAttrFlags,
+        util::{mount_bind, mount_fs, set_root_mount_propagation},
+    },
     path::PATH_MAX,
     unshare::{GidMap, UidMap},
-    xpath, XPath, XPathBuf,
+    xpath, XPathBuf,
 };
 
 bitflags! {
@@ -102,7 +109,7 @@ pub enum WordExpError {
     /// Process was aborted unexpectedly with signal.
     ProcessError(i32),
     /// Timeout error
-    TimeoutError(u64),
+    TimeoutError(u128),
 }
 
 /// Out of memory.
@@ -441,7 +448,7 @@ impl WordExp<'_> {
         let mut ret = Vec::new();
 
         loop {
-            if !sig && (err as i32 != 0 || epoch.elapsed() >= timeout) {
+            if !sig && (err as i32 != 0 || epoch.elapsed() >= timeout.into()) {
                 // a. Out of memory condition
                 // b. Timeout exceeded
                 // Send SIGKILL once, and fall-through to wait.
@@ -467,7 +474,7 @@ impl WordExp<'_> {
                         continue;
                     }
                     Err(ref e) if matches!(err2no(e), Errno::EAGAIN | Errno::EINTR) => {
-                        std::thread::sleep(Duration::from_millis(100));
+                        std::thread::sleep(Duration::from_millis(100).into());
                         continue;
                     }
                     Err(ref e) => {
@@ -646,30 +653,27 @@ impl WordExp<'_> {
             // SAFETY: remount rootfs as readonly,nosuid,nodev,nosymfollow
             // Careful here, unshare(2) may be available but mount(2) may not be,
             // so we must handle mount errors gracefully.
-            let mut flags: MsFlags = MsFlags::MS_BIND
-                | MsFlags::MS_REC
-                | MsFlags::MS_RDONLY
-                | MsFlags::MS_NOSUID
-                | MsFlags::MS_NODEV
-                | MS_NOSYMFOLLOW;
-            if mount(
-                Some("none"),
-                "/",
-                None::<&XPath>,
-                MsFlags::MS_PRIVATE | MsFlags::MS_REC,
-                None::<&XPath>,
-            )
-            .is_ok()
-            {
+            let mut flags = MountAttrFlags::MOUNT_ATTR_RDONLY
+                | MountAttrFlags::MOUNT_ATTR_NOSUID
+                | MountAttrFlags::MOUNT_ATTR_NODEV
+                | MountAttrFlags::MOUNT_ATTR_NOSYMFOLLOW;
+
+            if set_root_mount_propagation(MsFlags::MS_PRIVATE).is_ok() {
                 debug!("ctx": "expand",
-                    "msg": "set mount propagation to private in the new mount namespace");
-                mount(Some("/"), "/", Some("/"), flags, None::<&XPath>).expect("remount rootfs");
+                    "msg": "set mount propagation to private in new mount namespace");
+                open(
+                    "/",
+                    OFlag::O_CLOEXEC | OFlag::O_PATH | OFlag::O_DIRECTORY,
+                    Mode::empty(),
+                )
+                .and_then(|root| mount_bind(&root, &root, flags))
+                .expect("remount rootfs");
                 debug!("ctx": "expand",
-                    "msg": "remounted root with readonly, nosuid, nodev, and nosymfollow options in the new mount namespace");
+                    "msg": "remounted root with readonly, nosuid, nodev, and nosymfollow options in new mount namespace");
                 // SAFETY: mount private procfs
                 // pid=1 is required to exist before this.
-                flags.remove(MsFlags::MS_BIND | MsFlags::MS_REC | MS_NOSYMFOLLOW);
-                flags.insert(MsFlags::MS_NOEXEC);
+                flags.remove(MountAttrFlags::MOUNT_ATTR_NOSYMFOLLOW);
+                flags.insert(MountAttrFlags::MOUNT_ATTR_NOEXEC);
                 Self::mount_proc(flags);
             }
         }
@@ -833,8 +837,11 @@ impl WordExp<'_> {
             }
         }
 
-        // SAFETY: confine with seccomp.
-        // panics on errors.
+        // Set file size rlimits to zero, panic on errors.
+        #[expect(clippy::disallowed_methods)]
+        confine_rlimit(Resource::RLIMIT_FSIZE, None).expect("setrlimit(RLIMIT_FSIZE,0)");
+
+        // SAFETY: confine with seccomp, panics on errors.
         Self::confine_seccomp();
     }
 
@@ -873,13 +880,13 @@ impl WordExp<'_> {
 
     #[expect(clippy::cognitive_complexity)]
     #[expect(clippy::disallowed_methods)]
-    fn mount_proc(flags: MsFlags) {
+    fn mount_proc(flags: MountAttrFlags) {
         let mut stack = [0u8; MINI_STACK_SIZE];
         safe_clone(
             Box::new(move || -> isize {
                 // pid=1 here.
                 debug!("ctx": "expand",
-                    "msg": "started init process in the new pid namespace");
+                    "msg": "started init process in new pid namespace");
 
                 // SAFETY: set parent-death signal to SIGKILL
                 if set_pdeathsig(Some(Signal::SIGKILL)).is_err() {
@@ -893,16 +900,22 @@ impl WordExp<'_> {
                     .expect("block signals");
 
                 // SAFETY: mount private procfs, continue on errors.
-                match mount(
-                    Some("proc"),
+                match open(
                     "/proc",
-                    Some("proc"),
-                    flags,
-                    Some("hidepid=4"),
-                ) {
+                    OFlag::O_CLOEXEC | OFlag::O_PATH | OFlag::O_DIRECTORY,
+                    Mode::empty(),
+                )
+                .and_then(|proc| {
+                    mount_fs(
+                        OsStr::new("proc"),
+                        proc,
+                        flags,
+                        Some("hidepid=4,subset=pid"),
+                    )
+                }) {
                     Ok(_) => {
                         debug!("ctx": "expand",
-                            "msg": "mounted proc with hidepid=4 in the new mount namespace");
+                            "msg": "mounted proc with hidepid=4,subset=pid in new mount namespace");
                     }
                     Err(errno) => {
                         debug!("ctx": "expand",
@@ -911,7 +924,7 @@ impl WordExp<'_> {
                 };
 
                 // SAFETY: block until the parent-death signal kills us.
-                std::thread::sleep(Duration::MAX);
+                std::thread::sleep(std::time::Duration::MAX);
 
                 unreachable!();
             }),
